@@ -3,7 +3,7 @@
 use crate::{Agent, Consumer, Producer};
 
 use {
-    alloc::string::String,
+    alloc::string::{String, ToString},
     core::{
         convert::TryFrom,
         fmt::{self, Debug, Display, Formatter},
@@ -49,22 +49,42 @@ pub trait TryBlame<T> {
 
 /// The cause of an [`Agent`] failing to successfully complete an action upon a market.
 #[non_exhaustive]
-pub enum Fault<F: Flaws> {
+pub enum Fault<F>
+where
+    F: Flaws,
+{
     /// The action failed due to an insufficiency.
     Insufficiency(F::Insufficiency),
     /// The action failed due to a defect.
     Defect(F::Defect),
 }
 
-impl<F: Flaws> Fault<F> {
+impl<F> Fault<F>
+where
+    F: Flaws,
+{
     /// Returns if `self` is a defect.
     fn is_defect(&self) -> bool {
         matches!(*self, Self::Defect(_))
     }
+
+    /// If `self` is a defect, converts the defect into `W::Defect`; otherwise returns `self`.
+    fn map_defect<M, W>(self, mut m: M) -> Fault<W>
+    where
+        M: FnMut(F::Defect) -> W::Defect,
+        W: Flaws<Insufficiency = F::Insufficiency>,
+    {
+        match self {
+            Self::Insufficiency(insufficiency) => Fault::Insufficiency(insufficiency),
+            Self::Defect(defect) => Fault::Defect(m(defect)),
+        }
+    }
 }
 
-impl<F: Flaws, W: Flaws> Blame<Fault<W>> for Fault<F>
+impl<F, W> Blame<Fault<W>> for Fault<F>
 where
+    F: Flaws,
+    W: Flaws,
     W::Insufficiency: From<F::Insufficiency>,
     W::Defect: From<F::Defect>,
 {
@@ -78,8 +98,9 @@ where
     }
 }
 
-impl<F: Flaws> Clone for Fault<F>
+impl<F> Clone for Fault<F>
 where
+    F: Flaws,
     F::Insufficiency: Clone,
     F::Defect: Clone,
 {
@@ -91,15 +112,17 @@ where
     }
 }
 
-impl<F: Flaws> Copy for Fault<F>
+impl<F> Copy for Fault<F>
 where
+    F: Flaws,
     F::Insufficiency: Copy,
     F::Defect: Copy,
 {
 }
 
-impl<F: Flaws> Debug for Fault<F>
+impl<F> Debug for Fault<F>
 where
+    F: Flaws,
     F::Insufficiency: Debug,
     F::Defect: Debug,
 {
@@ -113,8 +136,9 @@ where
     }
 }
 
-impl<F: Flaws> Display for Fault<F>
+impl<F> Display for Fault<F>
 where
+    F: Flaws,
     F::Insufficiency: Display,
     F::Defect: Display,
 {
@@ -126,8 +150,9 @@ where
     }
 }
 
-impl<F: Flaws> PartialEq for Fault<F>
+impl<F> PartialEq for Fault<F>
 where
+    F: Flaws,
     F::Insufficiency: PartialEq,
     F::Defect: PartialEq,
 {
@@ -151,8 +176,10 @@ where
     }
 }
 
-impl<F: Flaws, W: Flaws> TryBlame<Fault<W>> for Fault<F>
+impl<F, W> TryBlame<Fault<W>> for Fault<F>
 where
+    F: Flaws,
+    W: Flaws,
     W::Insufficiency: TryFrom<F::Insufficiency>,
     W::Defect: TryFrom<F::Defect>,
 {
@@ -174,31 +201,57 @@ where
 
 /// The error thrown when the action of an [`Agent`] fails.
 pub struct Failure<F: Flaws> {
-    /// The name of the [`Agent`].
-    agent_name: String,
+    /// The description of the [`Agent`].
+    agent_description: String,
     /// The cause of the failure.
     fault: Fault<F>,
 }
 
-impl<F: Flaws> Failure<F> {
-    /// Creates a new [`Failure`] with the `agent_name`and `fault` that caused the failure.
-    pub(crate) fn new(fault: Fault<F>, agent_name: String) -> Self {
-        Self { agent_name, fault }
+impl<F> Failure<F>
+where
+    F: Flaws,
+{
+    /// Creates a new [`Failure`] with the description of `agent`and `fault` that caused the failure.
+    pub(crate) fn new<A>(agent: &A, fault: Fault<F>) -> Self
+    where
+        A: Display,
+    {
+        Self {
+            agent_description: agent.to_string(),
+            fault,
+        }
     }
 
     /// Returns if `self` was caused by a defect.
     pub fn is_defect(&self) -> bool {
         self.fault.is_defect()
     }
+
+    /// If `self` is a defect, converts the defect into `W::Defect`; otherwise returns `self`.
+    pub fn map_defect<M, W>(self, m: M) -> Failure<W>
+    where
+        M: FnMut(F::Defect) -> W::Defect,
+        W: Flaws<Insufficiency = F::Insufficiency>,
+    {
+        Failure {
+            agent_description: self.agent_description,
+            fault: self.fault.map_defect(m),
+        }
+    }
 }
 
-impl<F: Flaws, W: Flaws> Blame<Failure<W>> for Failure<F>
+impl<F, W> Blame<Failure<W>> for Failure<F>
 where
+    F: Flaws,
+    W: Flaws,
     W::Insufficiency: From<F::Insufficiency>,
     W::Defect: From<F::Defect>,
 {
     fn blame(self) -> Failure<W> {
-        Failure::new(self.fault.blame(), self.agent_name)
+        Failure {
+            agent_description: self.agent_description,
+            fault: self.fault.blame(),
+        }
     }
 }
 
@@ -210,7 +263,7 @@ where
     /// Writes the default debug format for `self`.
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         f.debug_struct("Failure")
-            .field("agent_name", &self.agent_name)
+            .field("agent_description", &self.agent_description)
             .field("fault", &self.fault)
             .finish()
     }
@@ -223,7 +276,7 @@ where
 {
     /// Writes "{name}: {fault}".
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
-        write!(f, "{}: {}", self.agent_name, self.fault)
+        write!(f, "{}: {}", self.agent_description, self.fault)
     }
 }
 
@@ -242,7 +295,7 @@ where
     F::Defect: PartialEq,
 {
     fn eq(&self, other: &Self) -> bool {
-        self.agent_name == other.agent_name && self.fault == other.fault
+        self.agent_description == other.agent_description && self.fault == other.fault
     }
 }
 
@@ -256,10 +309,13 @@ where
     #[throws(Self::Error)]
     fn try_blame(self) -> Failure<W> {
         match self.fault.try_blame() {
-            Ok(fault) => Failure::new(fault, self.agent_name),
+            Ok(fault) => Failure {
+                agent_description: self.agent_description,
+                fault,
+            },
             Err(error) => throw!(FailureConversionError {
                 error,
-                agent_name: self.agent_name
+                agent_description: self.agent_description
             }),
         }
     }
@@ -358,6 +414,90 @@ where
     }
 }
 
+/// The error thrown when a chain from a [`Consumer`] to a [`Producer`] fails to produce a good.
+#[non_exhaustive]
+pub enum Blockage<C, P, G>
+where
+    C: Flaws,
+    P: Flaws,
+{
+    /// The action failed due to a failure during consumption.
+    Consumption(Failure<C>),
+    /// The action failed due to a failure during production.
+    Production(Recall<P, G>),
+}
+
+impl<C, P, G> Debug for Blockage<C, P, G>
+where
+    C: Flaws,
+    C::Insufficiency: Debug,
+    C::Defect: Debug,
+    P: Flaws,
+    P::Insufficiency: Debug,
+    P::Defect: Debug,
+    G: Debug,
+{
+    /// Writes the default debug format for `self`.
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        match *self {
+            Self::Consumption(ref failure) => {
+                write!(f, "Blockage::Consumption({:?})", failure)
+            }
+            Self::Production(ref recall) => write!(f, "Blockage::Production({:?})", recall),
+        }
+    }
+}
+
+impl<C, P, G> From<Failure<C>> for Blockage<C, P, G>
+where
+    C: Flaws,
+    P: Flaws,
+{
+    fn from(failure: Failure<C>) -> Self {
+        Self::Consumption(failure)
+    }
+}
+
+impl<C, P, G> From<Recall<P, G>> for Blockage<C, P, G>
+where
+    C: Flaws,
+    P: Flaws,
+{
+    fn from(recall: Recall<P, G>) -> Self {
+        Self::Production(recall)
+    }
+}
+
+impl<C, P, G> PartialEq for Blockage<C, P, G>
+where
+    C: Flaws,
+    C::Insufficiency: PartialEq,
+    C::Defect: PartialEq,
+    P: Flaws,
+    P::Insufficiency: PartialEq,
+    P::Defect: PartialEq,
+    G: PartialEq,
+{
+    fn eq(&self, other: &Self) -> bool {
+        match *self {
+            Self::Consumption(ref my_failure) => {
+                if let Blockage::Consumption(ref their_failure) = *other {
+                    my_failure == their_failure
+                } else {
+                    false
+                }
+            }
+            Self::Production(ref my_recall) => {
+                if let Blockage::Production(ref their_recall) = *other {
+                    my_recall == their_recall
+                } else {
+                    false
+                }
+            }
+        }
+    }
+}
+
 /// The error thrown when `Fault::blame()` fails.
 #[non_exhaustive]
 pub enum FaultConversionError<F: Flaws, W: Flaws>
@@ -427,7 +567,7 @@ where
     /// The error that caused the failure.
     error: FaultConversionError<F, W>,
     /// The name of the [`Agent`] that experienced the failure.
-    agent_name: String,
+    agent_description: String,
 }
 
 impl<F: Flaws, W: Flaws> Debug for FailureConversionError<F, W>
@@ -440,7 +580,7 @@ where
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         f.debug_struct("FailureConversionError")
             .field("error", &self.error)
-            .field("agent_name", &self.agent_name)
+            .field("agent_description", &self.agent_description)
             .finish()
     }
 }
@@ -453,7 +593,7 @@ where
     <F::Defect as TryFrom<W::Defect>>::Error: Display,
 {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
-        write!(f, "in `{}`: {}", self.agent_name, self.error)
+        write!(f, "in `{}`: {}", self.agent_description, self.error)
     }
 }
 
